/* Copyright (C) Sportradar AG. See LICENSE for full license governing this code */
package com.sportradar.unifiedodds.example;

import java.util.Collection;
import java.util.HashMap;

import com.sportradar.unifiedodds.sdk.OddsFeedListener;
import com.sportradar.unifiedodds.sdk.OddsFeedSession;
import com.sportradar.unifiedodds.sdk.oddsentities.BetCancel;
import com.sportradar.unifiedodds.sdk.oddsentities.BetSettlement;
import com.sportradar.unifiedodds.sdk.oddsentities.BetStop;
import com.sportradar.unifiedodds.sdk.oddsentities.FixtureChange;
import com.sportradar.unifiedodds.sdk.oddsentities.Market;
import com.sportradar.unifiedodds.sdk.oddsentities.MarketBetSettlement;
import com.sportradar.unifiedodds.sdk.oddsentities.MarketOddsChange;
import com.sportradar.unifiedodds.sdk.oddsentities.MarketStatus;
import com.sportradar.unifiedodds.sdk.oddsentities.OddsChange;
import com.sportradar.unifiedodds.sdk.oddsentities.Outcome;
import com.sportradar.unifiedodds.sdk.oddsentities.OutcomeOdds;
import com.sportradar.unifiedodds.sdk.oddsentities.OutcomeResult;
import com.sportradar.unifiedodds.sdk.oddsentities.RollbackBetCancel;
import com.sportradar.unifiedodds.sdk.oddsentities.RollbackBetSettlement;
import com.sportradar.unifiedodds.sdk.sportentities.SportEvent;

public class MiniMTS implements OddsFeedListener {

    private static class Ticket {
        SportEvent competition;
        Market market;
        Outcome outcome;
        double odds;
        double amount;
    }

    private HashMap<SportEvent, HashMap<Market, HashMap<Outcome, Double>>> allOdds = new HashMap<>();
    private HashMap<SportEvent, Integer> acceptingOdds = new HashMap<>();
    private HashMap<SportEvent, Collection<Ticket>> allTickets = new HashMap<>();
    private long lastReceivedHB;

    @Override
    public void onOddsChange(OddsFeedSession sender, OddsChange<SportEvent> x) {
        SportEvent c = x.getEvent();
        HashMap<Market, HashMap<Outcome, Double>> allOddsForCompetition = allOdds.get(c);
        Collection<MarketOddsChange> oddsChanges = x.getOddsChanges();
        for (MarketOddsChange oddsChange : oddsChanges) {
            Market market = oddsChange.getMarket();
            HashMap<Outcome, Double> oddsForMarket = allOddsForCompetition.get(market);
            if (oddsChange.getStatus() != MarketStatus.Deactivated) {
                for (OutcomeOdds outcomeOdds : oddsChange.getOutcomeOdds()) {
                    System.out.println("Odds for " + c + " " + market + " " + outcomeOdds.getOutcome() + " is "
                                + outcomeOdds.getOdds());
                    oddsForMarket.put(outcomeOdds.getOutcome(), outcomeOdds.getOdds());
                }
            } else {
                // deactive odds for market X here
                for (Outcome outcome : oddsForMarket.keySet()) {
                    oddsForMarket.put(outcome, null);
                }
            }
        }
    }

    @Override
    public void onBetSettlement(OddsFeedSession sender, BetSettlement<SportEvent> clearBets) {
        SportEvent competition = clearBets.getEvent();
        Collection<Ticket> tickets = allTickets.get(competition);

        for (MarketBetSettlement clearBet : clearBets.getBetSettlements()) {
            Market market = clearBet.getMarket();
            for (OutcomeResult result : clearBet.getOutcomeResults()) {
                Outcome outcome = result.getOutcome();
                for (Ticket t : tickets) {
                    if (t.market.equals(market) && t.outcome.equals(outcome)) {
                        if (result.getRefundFactor() == 1.0) {
                            System.out.println("Ticket " + t + " refunded complete amount " + t.amount);
                        } else if (result.getRefundFactor() == 0.5) {
                            if (result.isWinning()) {
                                System.out.println("Ticket " + t + " is half refunded + half won: "
                                            + (t.amount + t.amount * t.odds) / 2);
                            } else {
                                System.out.println(
                                            "Ticket " + t + " is half refunded the rest is lost " + t.amount / 2);
                            }
                        } else if (result.getRefundFactor() == 0) {
                            if (result.isWinning()) {
                                System.out.println("Ticket " + t + " won and cash out is " + t.amount * t.odds);
                            } else {
                                System.out.println("Ticket " + t + " is a loss");
                            }
                        }
                    }
                }
                // if refundfactor == 1 pay out amount payed
                // if refundfactor == 0.5 && isWin pay out half + half
                // if refundfactor == 0.5 && !isWin pay out half paid only
                // if refundfactor == 0 && isWin pay out paid * odds
                // if refundfactor == 0 && !isWin tell customer loss
            }
        }
    }

    @Override
    public void onBetCancel(OddsFeedSession sender, BetCancel<SportEvent> betCancel) {
        // here find all tickets that match and repay and cancel

    }

    @Override
    public void onRollbackBetCancel(OddsFeedSession sender, RollbackBetCancel<SportEvent> rbc) {
        // here find all tickets that match - reactivate and take the amount again

    }

    public void onIncomingTick(Ticket ticket) {
        SportEvent competition = ticket.competition;
        Market market = ticket.market;
        Integer accepting = acceptingOdds.get(competition);
        if (accepting == null || accepting == 0) {
            throw new RuntimeException("Not accepting odds on this match at the moment");
        }
        // HashMap<Market, BetStatus> am = acceptingOddsForMarket.get(competition);
        double currentOdds = allOdds.get(competition).get(market).get(ticket.outcome);
        if (Math.abs(currentOdds - ticket.odds) < 0.0005) {
            // accept the ticket
            Collection<Ticket> tickets = allTickets.get(competition);
            tickets.add(ticket);
        }
    }

    public void heartbeatCheck() {
        if (System.currentTimeMillis() - lastReceivedHB > 5000) {
            // Have not received a heartbeat in 5 seconds... stop everything!?
            System.err.println("Stream appears to have stalled . Stopping everything for safety");
        }
    }

    @Override
    public void onRollbackBetSettlement(OddsFeedSession sender,
                RollbackBetSettlement<SportEvent> rollbackBetSettlement) {
        // TODO Auto-generated method stub

    }

    @Override
    public void onFixtureChange(OddsFeedSession sender, FixtureChange<SportEvent> fixtureChange) {
        // TODO Auto-generated method stub

    }

    @Override
    public void onBetStop(OddsFeedSession sender, BetStop<SportEvent> betStop) {
        // TODO Auto-generated method stub

    }

    @Override
    public void onUnparseableMessage(OddsFeedSession sender, byte[] rawMessage, SportEvent event) {
        // TODO Auto-generated method stub

    }
}
