/* Copyright (C) Sportradar AG. See LICENSE for full license governing this code */
package com.sportradar.unifiedodds.example.basic;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sportradar.unifiedodds.sdk.MarketDescriptionManager;
import com.sportradar.unifiedodds.sdk.OddsFeedListener;
import com.sportradar.unifiedodds.sdk.OddsFeedSession;
import com.sportradar.unifiedodds.sdk.oddsentities.BetCancel;
import com.sportradar.unifiedodds.sdk.oddsentities.BetSettlement;
import com.sportradar.unifiedodds.sdk.oddsentities.BetStop;
import com.sportradar.unifiedodds.sdk.oddsentities.FixtureChange;
import com.sportradar.unifiedodds.sdk.oddsentities.MarketBetSettlement;
import com.sportradar.unifiedodds.sdk.oddsentities.MarketOddsChange;
import com.sportradar.unifiedodds.sdk.oddsentities.MarketStatus;
import com.sportradar.unifiedodds.sdk.oddsentities.OddsChange;
import com.sportradar.unifiedodds.sdk.oddsentities.OutcomeOdds;
import com.sportradar.unifiedodds.sdk.oddsentities.OutcomeResult;
import com.sportradar.unifiedodds.sdk.oddsentities.RollbackBetCancel;
import com.sportradar.unifiedodds.sdk.oddsentities.RollbackBetSettlement;
import com.sportradar.unifiedodds.sdk.sportentities.SportEvent;

public class MessageListener implements OddsFeedListener {
    private Logger logger = LoggerFactory.getLogger("basic_example");

    @Override
    public void onOddsChange(OddsFeedSession sender, OddsChange<SportEvent> oddsChanges) {
        logger.info("Received odds change for: " + oddsChanges.getEvent());

        MarketDescriptionManager mktDescMgr = sender.getMarketDescriptionManager();
        // Now loop through the odds for each market
        for (MarketOddsChange marketOdds : oddsChanges.getOddsChanges()) {
            // Now loop through the outcomes within this particular market
            String marketDescription = mktDescMgr.getMarketDescription(oddsChanges.getEvent(), marketOdds.getMarket());
            logger.info("Received odds information for : " + marketDescription);
            logger.info("Market status is : " + marketOdds.getStatus());

            // If the market is active printout odds for all outcomes
            if (marketOdds.getStatus() == MarketStatus.Active) {
                for (OutcomeOdds outcomeOdds : marketOdds.getOutcomeOdds()) {
                    String outcomeDesc = sender.getMarketDescriptionManager().getOutcomeDescription(
                                oddsChanges.getEvent(), marketOdds.getMarket(), outcomeOdds.getOutcome());
                    logger.info("Outcome " + outcomeDesc + " has odds " + outcomeOdds.getOdds() + " "
                                + outcomeOdds.getProbability());
                }
            }
        }
    }

    @Override
    public void onBetStop(OddsFeedSession sender, BetStop<SportEvent> betStop) {
        logger.info("Received betstop for sport event " + betStop.getEvent());
    }

    @Override
    public void onBetSettlement(OddsFeedSession sender, BetSettlement<SportEvent> clearBets) {
        logger.info("Received bet settlement for sport event " + clearBets.getEvent());

        // Iterate through the betsettlements for each market
        for (MarketBetSettlement marketSettlement : clearBets.getBetSettlements()) {
            // Then iterate through the result for each outcome (win or loss)
            for (OutcomeResult result : marketSettlement.getOutcomeResults()) {
                if (result.isWinning())
                    logger.info("Outcome " + result.getOutcome().getId() + " is a win");
                else
                    logger.info("Outcome " + result.getOutcome().getId() + " is a loss");
            }
        }
    }

    @Override
    public void onRollbackBetSettlement(OddsFeedSession sender,
                RollbackBetSettlement<SportEvent> rollbackBetSettlement) {
        logger.info("Received rollback betsettlement for sport event " + rollbackBetSettlement.getEvent());
    }

    @Override
    public void onBetCancel(OddsFeedSession sender, BetCancel<SportEvent> betCancel) {
        logger.info("Received bet cancel for sport event " + betCancel.getEvent());
    }

    @Override
    public void onRollbackBetCancel(OddsFeedSession sender, RollbackBetCancel<SportEvent> rbBetCancel) {
        logger.info("Received rollback betcancel for sport event " + rbBetCancel.getEvent());
    }

    @Override
    public void onFixtureChange(OddsFeedSession sender, FixtureChange<SportEvent> fixtureChange) {
        logger.info("Received fixture change for sport event " + fixtureChange.getEvent());
    }

    @Override
    public void onUnparseableMessage(OddsFeedSession sender, byte[] rawMessage, SportEvent event) {
        // TODO Auto-generated method stub

    }

}
