/* Copyright (C) Sportradar AG. See LICENSE for full license governing this code */
package com.sportradar.unifiedodds.example.recorder;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import com.sportradar.unifiedodds.sdk.InitException;
import com.sportradar.unifiedodds.sdk.MessageInterest;
import com.sportradar.unifiedodds.sdk.OddsFeedConfiguration;
import com.sportradar.unifiedodds.sdk.OddsFeedListener;
import com.sportradar.unifiedodds.sdk.OddsFeedSession;
import com.sportradar.unifiedodds.sdk.OddsFeedSessionBuilder;
import com.sportradar.unifiedodds.sdk.SDKGlobalEventsListener;
import com.sportradar.unifiedodds.sdk.oddsentities.BetCancel;
import com.sportradar.unifiedodds.sdk.oddsentities.BetSettlement;
import com.sportradar.unifiedodds.sdk.oddsentities.BetStop;
import com.sportradar.unifiedodds.sdk.oddsentities.FixtureChange;
import com.sportradar.unifiedodds.sdk.oddsentities.OddsChange;
import com.sportradar.unifiedodds.sdk.oddsentities.ProductDown;
import com.sportradar.unifiedodds.sdk.oddsentities.RollbackBetCancel;
import com.sportradar.unifiedodds.sdk.oddsentities.RollbackBetSettlement;
import com.sportradar.unifiedodds.sdk.oddsentities.SDKFeedMessage;
import com.sportradar.unifiedodds.sdk.sportentities.SportEvent;

/**
 * This is more of a tool than an example. This tool can be used to record all messages received and
 * store them in a file (and then replayed to your own local RabbitMQ server for testing - see the
 * Sender example)
 * 
 * Usage: java -Duf.accesstoken=<mytoken> MessageRecorder <file-name-where-to-store-messages>
 */
public class MessageRecorder implements OddsFeedListener, SDKGlobalEventsListener {
    private OutputStream outputStream;
    private OddsFeedSession session;

    public static void main(String[] args) throws InitException, IOException, InterruptedException {
        MessageRecorder recorder = new MessageRecorder();
        recorder.init(args[0]);

        System.out.println("Now recording for 10 minutes to " + args[0]);
        Thread.sleep(1000 * 60 * 10);
    }

    public void init(String fileName) throws InitException, IOException {
        OddsFeedConfiguration config = new OddsFeedConfiguration();
        OddsFeedSessionBuilder builder = OddsFeedSessionBuilder.init(this, config);
        session = builder.setListener(this).setMessageInterest(MessageInterest.AllMessages).build();
        outputStream = new FileOutputStream(fileName);
    }

    @Override
    public void onBetCancel(OddsFeedSession sender, BetCancel<SportEvent> betCancel) {
        store(betCancel);
    }

    @Override
    public void onRollbackBetCancel(OddsFeedSession sender, RollbackBetCancel<SportEvent> rbBetCancel) {
        store(rbBetCancel);
    }

    @Override
    public void onFixtureChange(OddsFeedSession sender, FixtureChange<SportEvent> fixtureChange) {
        store(fixtureChange);
    }

    @Override
    public void onProductDown(ProductDown productDown) {

    }

    @Override
    public void onOddsChange(OddsFeedSession sender, OddsChange<SportEvent> oddsChanges) {
        store(oddsChanges);
    }

    @Override
    public void onBetStop(OddsFeedSession sender, BetStop<SportEvent> betStop) {
        store(betStop);
    }

    @Override
    public void onBetSettlement(OddsFeedSession sender, BetSettlement<SportEvent> clearBets) {
        store(clearBets);
    }

    @Override
    public void onRollbackBetSettlement(OddsFeedSession sender,
                RollbackBetSettlement<SportEvent> rollbackBetSettlement) {
        store(rollbackBetSettlement);
    }

    @Override
    public void onConnectionDown() {

    }

    private void store(SDKFeedMessage msg) {
        try {
            outputStream.write(msg.getRawMessage());
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onUnparseableMessage(OddsFeedSession sender, byte[] rawMessage, SportEvent event) {
        // TODO Auto-generated method stub

    }
}
