/* Copyright (C) Sportradar AG. See LICENSE for full license governing this code */
package com.sportradar.unifiedodds.restfuloddscache;

import java.io.IOException;
import java.io.PrintWriter;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringEscapeUtils;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.AbstractHandler;
import org.json.JSONObject;
import org.json.XML;

import com.sportradar.unifiedodds.sdk.MarketDescriptionManager;
import com.sportradar.unifiedodds.sdk.SportsInfo;
import com.sportradar.unifiedodds.sdk.oddsentities.MarketStatus;
import com.sportradar.unifiedodds.sdk.oddsentities.Outcome;
import com.sportradar.unifiedodds.sdk.oddsentities.Product;
import com.sportradar.unifiedodds.sdk.sportentities.Fixture;
import com.sportradar.unifiedodds.sdk.sportentities.HomeAway;
import com.sportradar.unifiedodds.sdk.sportentities.Score;
import com.sportradar.unifiedodds.sdk.sportentities.SportEvent;
import com.sportradar.unifiedodds.sdk.sportentities.SportEventStatus;
import com.sportradar.unifiedodds.sdk.sportentities.Team;
import com.sportradar.unifiedodds.sdk.sportentities.Tournament;

/**
 * This class is an extremely simple http-request-handler that handles some hardcoded requests and
 * responds with the current odds for those requests
 * 
 */
class HTTPOddsRequestHandler extends AbstractHandler {

	private com.sportradar.unifiedodds.restfuloddscache.OddsListener listener;

	private MarketDescriptionManager marketDescManager;

	private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-DD'T'HH:mm:ssXXX");

	private SportsInfo sportsManager;

	public HTTPOddsRequestHandler(com.sportradar.unifiedodds.restfuloddscache.OddsListener listener, MarketDescriptionManager marketDescManager, SportsInfo sportsInfo) {
		this.listener = listener;
		this.marketDescManager = marketDescManager;
		this.sportsManager = sportsInfo;
	}

	@Override
	public void handle(String target, Request base, HttpServletRequest req, HttpServletResponse rsp)
			throws IOException, ServletException {
		if (target.startsWith("/log/")) {
			String path = target.substring(5);
			String id = path;
			String res = listener.getLog(id);
			rsp.setContentType("text;charset=utf-8");
			rsp.setStatus(HttpServletResponse.SC_OK);
			base.setHandled(true);
			rsp.getWriter().println(res);
			return;
		}

		else if (target.startsWith("/odds/")) {
			String display = req.getParameter("display");
			boolean showProbs = "true".equals(req.getParameter("showProbs"));
			// Handles /odds/sr:match:1234/odds.xml
			String path = target.substring(6);
			int idx = path.indexOf("/");
			String id = path;
			if (idx != -1) {
				id = path.substring(0, idx);
			}
			MatchOdds ofm = listener.getOddsForMatch(id);
			if (ofm == null)
				return;
			rsp.setContentType("text;charset=utf-8");
			rsp.setStatus(HttpServletResponse.SC_OK);
			base.setHandled(true);
			oddsResponse(ofm, base, rsp, display, showProbs);
		}
		else if (target.startsWith("/oddsjson/")) {
			// Handles /odds/sr:match:1234/odds.xml
			String path = target.substring(10);
			int idx = path.indexOf("/");
			String id = path;
			if (idx != -1) {
				id = path.substring(0, idx);
			}
			MatchOdds ofm = listener.getOddsForMatch(id);
			if (ofm == null)
				return;
			rsp.setContentType("text;charset=utf-8");
			rsp.setStatus(HttpServletResponse.SC_OK);
			base.setHandled(true);
			oddsResponseJson(ofm, base, rsp);
		}
		else if (target.startsWith("/pretty/")) {
			String path = target.substring(8);
			int idx = path.indexOf("/");
			String id = path;
			if (idx != -1) {
				id = path.substring(0, idx);
			}
			MatchOdds ofm = listener.getOddsForMatch(id);
			if (ofm == null)
				return;
			rsp.setContentType("text/html;charset=utf-8");
			rsp.setStatus(HttpServletResponse.SC_OK);
			base.setHandled(true);
			prettyPrint(ofm, base, rsp, req);
		}
		else if (target.equals("/unsettled_events")) {
			List<MatchOdds> ls = listener.getUnsettledEvents();
			rsp.setStatus(HttpServletResponse.SC_OK);
			base.setHandled(true);
			if (ls == null || ls.size() == 0) {
				rsp.getWriter().println("none");
			}
			for (MatchOdds mo : ls) {
				oddsResponse(mo, base, rsp, null, false);
			}
		}
		else if (target.equals("/recently_changed_events")) {
			rsp.setContentType("text;charset=utf-8");
			rsp.setStatus(HttpServletResponse.SC_OK);
			base.setHandled(true);
			PrintWriter w = rsp.getWriter();
			Set<String> ids = listener.getRecentlyChangedEventIds();
			for (String id : ids) {
				SportEvent e = sportsManager.getSportEvent(id);
				MatchOdds mo = listener.getOddsForMatch(id);
				if (mo != null) {
					e = mo.getSportEvent();
					Team home = e.getFixture().getHomeTeam();
					Team away = e.getFixture().getAwayTeam();
					String hs = home == null ? "?" : home.getName();
					String as = away == null ? "?" : away.getName();
					w.printf("%-17s||%s:%s:%s %s-%s\n", e.getId(), e.getFixture().getTournament().getSportName(), e.getFixture().getTournament().getSportCategoryName(), e.getFixture().getTournament().getName(), hs, as);
				}
				else {
					Team home = e.getFixture().getHomeTeam();
					Team away = e.getFixture().getAwayTeam();
					String hs = home == null ? "?" : home.getName();
					String as = away == null ? "?" : away.getName();
					w.printf("%17s*|%s:%s:%s %s-%s (NO ODDS)\n", e.getId(), e.getFixture().getTournament().getSportName(), e.getFixture().getTournament().getSportCategoryName(), e.getFixture().getTournament().getName(), hs, as);
				}
			}
		}
		else if (target.equals("/log_queue_age")) {
			List<Integer> topAges = listener.getLogQueueAges();
			rsp.setContentType("text;charset=utf-8");
			rsp.setStatus(HttpServletResponse.SC_OK);
			base.setHandled(true);
			PrintWriter w = rsp.getWriter();
			for (Integer age : topAges) {
				w.println(age);
			}
		}
	}

	private void prettyPrint(MatchOdds ofm, Request base, HttpServletResponse rsp, HttpServletRequest req)
			throws IOException {
		String marketsStr = req.getParameter("markets");
		List<Integer> markets = new LinkedList<>();
		if (marketsStr != null) {
			String[] ms = marketsStr.split(",");
			for (String s : ms) {
				markets.add(Integer.parseInt(s.trim()));
			}
		}
		PrintWriter pw = rsp.getWriter();
		pw.println("<HEAD><TITLE>Odds for Match</TITLE></HEAD><BODY>");
		pw.println("<TABLE>");
		ArrayList<MarketOdds> mls = new ArrayList<MarketOdds>(ofm.getMarkets());
		mls.sort(marketOddsComparator);
		for (MarketOdds mo : mls) {
			if (markets.size() > 0) {
				if (!markets.contains(mo.getMarket().getId()))
					continue;
			}
			pw.println("<TR>");
			pw.printf("<TD>%d</TD><TD>%s</TD><TD>%s</TD>", mo.getMarket().getId(), mo.getMarket().getMarketDescription(), mo.getStatus());
			pw.println("<TD><TABLE>");
			for (Map.Entry<Outcome, Double> oo : mo.getOutcomes()) {
				pw.println("<TR>");
				pw.printf("<TD>%s</TD><TD>%s</TD>", oo.getKey().getId(), oo.getKey().getDescription());
				if (mo.getStatus() == MarketStatus.Active || mo.getStatus() == MarketStatus.Suspended) {
					pw.printf("<TD>%f</TD>", oo.getValue());
				}
				pw.println("</TR>");
			}
			pw.println("</TABLE></TD>");
			pw.println("</TR>");
		}
		pw.println("</TABLE>");
		pw.println("</BODY>");
	}

	private Comparator<MarketOdds> marketOddsComparator = new Comparator<MarketOdds>() {
		@Override
		public int compare(MarketOdds o1, MarketOdds o2) {
			return o1.getMarket().getId() - o2.getMarket().getId();
		}
	};

	private Comparator<Entry<Outcome, Double>> outcomeSorter = new Comparator<Entry<Outcome, Double>>() {
		@Override
		public int compare(Entry<Outcome, Double> o1, Entry<Outcome, Double> o2) {
			return o1.getKey().getId().compareTo(o2.getKey().getId());
		}
	};

	/**
	 * Print the odds for a provided match in XML-format - very raw!
	 * 
	 * @param matchOdds - the holder object containing all market odds for a certain match
	 * @param base - the original HTTP-Request
	 * @param rsp - the HTTP-response that is getting prepared
	 * @throws IOException - throw if the writer fails
	 */
	private void oddsResponse(MatchOdds matchOdds, Request base, HttpServletResponse rsp, String displayTheseMarketStatuses, boolean showProbs) throws IOException {
		MarketStatus[] onlyDisplay = MarketStatus.values();
		if (displayTheseMarketStatuses != null) {
			onlyDisplay = new MarketStatus[1];
			try {
				onlyDisplay[0] = MarketStatus.valueOf(displayTheseMarketStatuses);
			}
			catch (Exception e) {
				onlyDisplay = MarketStatus.values();
			}
		}
		PrintWriter w = rsp.getWriter();

		w.printf("<odds_information>\n");

		SportEvent event = matchOdds.getSportEvent();
		printSportEventInfo(w, event);

		w.printf("  <odds>\n");
		ArrayList<MarketOdds> mls = new ArrayList<MarketOdds>(matchOdds.getMarkets());
		mls.sort(marketOddsComparator);
		long now = System.currentTimeMillis();
		for (MarketOdds mo : mls) {
			MarketStatus status = mo.getStatus();
			boolean found = false;
			for (MarketStatus ms : onlyDisplay) {
				if (ms == status) {
					found = true;
					break;
				}
			}
			if (!found)
				continue;
			printMarket(w, now, mo, event, showProbs);
		}
		w.println("  </odds>");
		w.println("</odds_information>");
	}

	/**
	 * Print the odds for a provided match in JSON-format - very raw!
	 * 
	 * @param matchOdds - the holder object containing all market odds for a certain match
	 * @param base - the original HTTP-Request
	 * @param rsp - the HTTP-response that is getting prepared
	 * @throws IOException - throw if the writer fails
	 */
	private void oddsResponseJson(MatchOdds matchOdds, Request base, HttpServletResponse rsp) throws IOException {

		PrintWriter w = rsp.getWriter();

		StringBuilder xml = getOddsResponse(matchOdds, true);
		JSONObject xmlJSONObj = XML.toJSONObject(xml.toString());

		w.println(xmlJSONObj.toString());
	}

	public StringBuilder getOddsResponse(MatchOdds matchOdds, boolean showProbs) throws IOException {
		MarketStatus[] onlyDisplay = MarketStatus.values();

		StringBuilder sb = new StringBuilder();

		sb.append(String.format("<odds_information>\n"));

		SportEvent event = matchOdds.getSportEvent();

		sb.append(getSportEventInfo(event));

		sb.append(String.format("  <odds>\n"));
		ArrayList<MarketOdds> mls = new ArrayList<MarketOdds>(matchOdds.getMarkets());
		mls.sort(marketOddsComparator);
		long now = System.currentTimeMillis();
		for (MarketOdds mo : mls) {
			sb.append(getMarket(now, mo, event, showProbs));
		}
		sb.append(String.format("  </odds>"));
		sb.append(String.format("</odds_information>"));

		return sb;
	}

	/**
	 * Print some info about the specified SportEvent and its current status (very raw and
	 * incomplete)
	 * 
	 * @param w - the writer to write the results to
	 * @param event - the SportEvent to print
	 */
	private void printSportEventInfo(PrintWriter w, SportEvent event) {
		w.print(getSportEventInfo(event));
	}

	public StringBuilder getSportEventInfo(SportEvent event) {
		StringBuilder sb = new StringBuilder();
		Fixture fixture = event.getFixture();
		Tournament t = fixture.getTournament();
		if (fixture.getStartTime() == null) {

			sb.append(String.format("  <sport_event id=\"%s\" scheduled=\"unknown\">\n", event.getId()));
			sb.append(String.format("    <tournament id=\"%s\" name=\"%s\">\n", t.getId(), t.getName()));
			sb.append(String.format("      <sport id=\"0\" name=\"%s\"/>\n", t.getSportId(), t.getSportName()));
			sb.append(String.format("      <category id=\"%s\" name=\"%s\"/>\n", t.getSportCategoryId(), t.getSportCategoryName()));
			sb.append(String.format("    </tournament>\n"));
			sb.append(String.format("    <competitors>\n"));
			sb.append(String.format("    </competitors>\n"));
			sb.append(String.format("  </sport_event>\n"));

		}
		else {
			sb.append(String.format("  <sport_event id=\"%s\" scheduled=\"%s\">\n", event.getId(), dateFormat.format(fixture.getStartTime())));
			sb.append(String.format("    <tournament id=\"%s\" name=\"%s\">\n", t.getId(), t.getName()));
			sb.append(String.format("      <sport id=\"%s\" name=\"%s\"/>\n", t.getSportId(), t.getSportName()));
			sb.append(String.format("      <category id=\"%s\" name=\"%s\"/>\n", t.getSportCategoryId(), t.getSportCategoryName()));
			sb.append(String.format("    </tournament>\n"));
			sb.append(String.format("    <competitors>\n"));
			sb.append(String.format("       <competitor id=\"%s\" name=\"%s\" qualifier=\"home\"/>\n", fixture.getHomeTeam().getId(), fixture.getHomeTeam().getName()));
			sb.append(String.format("       <competitor id=\"%s\" name=\"%s\" qualifier=\"away\"/>\n", fixture.getAwayTeam().getId(), fixture.getAwayTeam().getName()));
			sb.append(String.format("    </competitors>\n"));
			sb.append(String.format("  </sport_event>\n"));
		}
		if (event.getStatus() == SportEventStatus.Live) {
			BigDecimal hsBD = event.getCompetitorStatusAttribute(HomeAway.Home, "score");
			BigDecimal asBD = event.getCompetitorStatusAttribute(HomeAway.Away, "score");
			int hs = hsBD != null ? hsBD.intValue() : 0;
			int as = asBD != null ? asBD.intValue() : 0;
			sb.append(String.format("  <sport_event_status status=\"live\" match_status=\"%s\" home_score=\"%d\" away_score=\"%d\">\n", event.getMatchStatusDescription(), hs, as));
			if (event.getPlayedTime() != null) {
				sb.append(String.format("    <clock time=\"%s\"", event.getPlayedTime()));
				String x = (String)event.getEventStatusAttribute("clock_remaining_time");
				String y = (String)event.getEventStatusAttribute("clock_remaining_time_in_period");
				if (x != null) {
					sb.append(String.format(" remaining_time=\"%s\"", x));
				}
				if (y != null) {
					sb.append(String.format(" remaining_time_in_period=\"%s\"", y));
				}
				sb.append(String.format("/>\n"));
			}
			List<Score> ls = event.getPeriodScores();
			if (ls != null) {
				sb.append(String.format("    <period_scores>\n"));
				int pn = 1;
				for (Score s : ls) {
					sb.append(String.format("      <period number=\"%d\" home_score=\"%d\" away_score=\"%d\"/>\n", pn, s.getHomeScore(), s.getAwayScore()));
					pn++;
				}
				sb.append(String.format("    </period_scores>\n"));
			}
			sb.append(String.format("  </sport_event_status>\n"));
		}
		else {
			if (event.getStatus() != null)
				sb.append(String.format("  <sport_event_status status=\"%s\"/>\n", event.getStatus().toString().toLowerCase()));
		}

		return sb;
	}

	void printMarket(PrintWriter w, long now, MarketOdds mo, SportEvent event, boolean showProbs) {
		w.print(getMarket(now, mo, event, showProbs));
	}

	public StringBuilder getMarket(long now, MarketOdds mo, SportEvent event, boolean showProbs) {
		StringBuilder sb = new StringBuilder();
		long noChangeSince = now - mo.getLastUpdateTime();
		int noChangeSinceSec = (int)noChangeSince / 1000;
		int noChangeSinceMin = noChangeSinceSec / 60;
		noChangeSinceSec = noChangeSinceSec % 60;
		Product product = mo.getLastProduct();

		if (product == null)
			product = Product.MTS;
		sb.append(String.format("    <market id=\"%d\" name=\"%s\" status=\"%s\"", mo.getMarket().getId(), StringEscapeUtils.escapeXml11(marketDescManager.getMarketDescription(event, mo.getMarket())), mo.getStatus().toString().toLowerCase()));
		if (displaySpecifiers() && mo.getMarket().getSpecifiers().size() > 0) {
			boolean first = true;
			for (String spec : mo.getMarket().getSpecifiers()) {
				String value = mo.getMarket().getSpecifierValue(spec);
				if (first) {
					sb.append(String.format(" specifiers=\""));
					first = false;
				}
				else
					sb.append(String.format("|"));
				sb.append(String.format("%s=%s", spec, value));
			}
			sb.append(String.format("\""));
		}
		// w.printf(" favourite=\"true\"");
		sb.append(String.format(" generated_by=\"%s\" changed_ago=\"%02d:%02d\">\n", product.toString().toLowerCase(), noChangeSinceMin, noChangeSinceSec));
		LinkedList<Entry<Outcome, Double>> ls = new LinkedList<Entry<Outcome, Double>>(mo.getOutcomes());
		ls.sort(outcomeSorter);
		for (Entry<Outcome, Double> oo : ls) {
			Outcome o = oo.getKey();
			String odesc = marketDescManager.getOutcomeDescription(event, mo.getMarket(), o);
			if (mo.getStatus() == MarketStatus.Settled) {
				Double value = oo.getValue();
				if (Math.abs(value - 1.0) < 0.1)
					sb.append(String.format("        <outcome id=\"%s\" name=\"%s\" result=\"won\"/>\n", o.getId(), StringEscapeUtils.escapeXml11(odesc)));
				else if (Math.abs(value) < 0.1)
					sb.append(String.format("        <outcome id=\"%s\" name=\"%s\" result=\"lost\"/>\n", o.getId(), StringEscapeUtils.escapeXml11(odesc)));
			}
			else {
				Double value = oo.getValue();
				Double probs = showProbs ? mo.getProbs(o) : null;
				if (probs != null) {
					sb.append(String.format("        <outcome id=\"%s\" name=\"%s\" odds=\"%.2f\" probs=\"%.7f\"/>\n", o.getId(), StringEscapeUtils.escapeXml11(odesc), value, probs));
				}
				else {
					sb.append(String.format("        <outcome id=\"%s\" name=\"%s\" odds=\"%.2f\"/>\n", o.getId(), StringEscapeUtils.escapeXml11(odesc), value));
				}
			}
		}
		sb.append(String.format("    </market>\n"));

		return sb;
	}

	private boolean displaySpecifiers() {
		return true;
	}

}
